package javax.xml.ws.spi;

import java.util.Iterator;
import java.lang.reflect.Method;
import java.io.IOException;
import java.io.Closeable;
import java.io.InputStream;
import java.io.FileInputStream;
import java.util.Properties;
import java.io.File;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import javax.xml.ws.WebServiceException;

class FactoryFinder
{
    private static final String OSGI_SERVICE_LOADER_CLASS_NAME = "com.sun.org.glassfish.hk2.osgiresourcelocator.ServiceLoader";
    
    FactoryFinder() {
    }
    
    private static Object newInstance(String className, ClassLoader classLoader) {
        try {
            Class<Object> spiClass = safeLoadClass(className, classLoader);
            return spiClass.newInstance();
        }
        catch (ClassNotFoundException x) {
            throw new WebServiceException("Provider " + className + " not found", x);
        }
        catch (Exception x2) {
            throw new WebServiceException("Provider " + className + " could not be instantiated: " + x2, x2);
        }
    }
    
    static Object find(String factoryId, String fallbackClassName) {
        if (isOsgi()) {
            return lookupUsingOSGiServiceLoader(factoryId);
        }
        ClassLoader classLoader;
        try {
            classLoader = Thread.currentThread().getContextClassLoader();
        }
        catch (Exception x) {
            throw new WebServiceException(x.toString(), x);
        }
        String serviceId = "META-INF/services/" + factoryId;
        BufferedReader rd = null;
        try {
            InputStream is;
            if (classLoader == null) {
                is = ClassLoader.getSystemResourceAsStream(serviceId);
            }
            else {
                is = classLoader.getResourceAsStream(serviceId);
            }
            if (is != null) {
                rd = new BufferedReader(new InputStreamReader(is, "UTF-8"));
                String factoryClassName = rd.readLine();
                if (factoryClassName != null && !"".equals(factoryClassName)) {
                    return newInstance(factoryClassName, classLoader);
                }
            }
        }
        catch (Exception ex) {}
        finally {
            close(rd);
        }
        FileInputStream inStream = null;
        try {
            String javah = System.getProperty("java.home");
            String configFile = javah + File.separator + "lib" + File.separator + "jaxws.properties";
            File f = new File(configFile);
            if (f.exists()) {
                Properties props = new Properties();
                inStream = new FileInputStream(f);
                props.load(inStream);
                String factoryClassName2 = props.getProperty(factoryId);
                return newInstance(factoryClassName2, classLoader);
            }
        }
        catch (Exception ex2) {}
        finally {
            close(inStream);
        }
        try {
            String systemProp = System.getProperty(factoryId);
            if (systemProp != null) {
                return newInstance(systemProp, classLoader);
            }
        }
        catch (SecurityException ex3) {}
        if (fallbackClassName == null) {
            throw new WebServiceException("Provider for " + factoryId + " cannot be found", null);
        }
        return newInstance(fallbackClassName, classLoader);
    }
    
    private static void close(Closeable closeable) {
        if (closeable != null) {
            try {
                closeable.close();
            }
            catch (IOException ex) {}
        }
    }
    
    private static Class safeLoadClass(String className, ClassLoader classLoader) throws ClassNotFoundException {
        try {
            SecurityManager s = System.getSecurityManager();
            if (s != null) {
                int i = className.lastIndexOf(46);
                if (i != -1) {
                    s.checkPackageAccess(className.substring(0, i));
                }
            }
            if (classLoader == null) {
                return Class.forName(className);
            }
            return classLoader.loadClass(className);
        }
        catch (SecurityException se) {
            if ("com.sun.xml.internal.ws.spi.ProviderImpl".equals(className)) {
                return Class.forName(className);
            }
            throw se;
        }
    }
    
    private static boolean isOsgi() {
        try {
            Class.forName("com.sun.org.glassfish.hk2.osgiresourcelocator.ServiceLoader");
            return true;
        }
        catch (ClassNotFoundException ex) {
            return false;
        }
    }
    
    private static Object lookupUsingOSGiServiceLoader(String factoryId) {
        try {
            Class<Object> serviceClass = Class.forName(factoryId);
            Class[] args = { serviceClass };
            Class<Object> target = Class.forName("com.sun.org.glassfish.hk2.osgiresourcelocator.ServiceLoader");
            Method m = target.getMethod("lookupProviderInstances", new Class[] { Class.class });
            Iterator<Object> iter = ((Iterable)m.invoke(null, (Object[])args)).iterator();
            return iter.hasNext() ? iter.next() : null;
        }
        catch (Exception ignored) {
            return null;
        }
    }
}
